==> imputegap-1.0.9-py3-none-any.whl: imputegap/wrapper/AlgoPython/BRITS/models/brits_i.py <==
import torch
import torch.nn as nn

from torch.autograd import Variable
from . import rits_i

class Model(nn.Module):
    def __init__(self, batch_size, nbr_features, hidden_layers, seq_length):
        super(Model, self).__init__()
        self.batch_size = batch_size
        self.nbr_features = nbr_features
        self.hidden_layers = hidden_layers
        self.seq_length = seq_length
        self.build()

    def build(self):
        self.rits_f = rits_i.Model(self.batch_size, self.nbr_features, self.hidden_layers, self.seq_length)
        self.rits_b = rits_i.Model(self.batch_size, self.nbr_features, self.hidden_layers, self.seq_length)

    def forward(self, data):
        ret_f = self.rits_f(data, 'forward')
        ret_b = self.reverse(self.rits_b(data, 'backward'))

        ret = self.merge_ret(ret_f, ret_b)

        return ret

    def merge_ret(self, ret_f, ret_b):
        loss_f = ret_f['loss']
        loss_b = ret_b['loss']
        loss_c = self.get_consistency_loss(ret_f['imputations'], ret_b['imputations'])

        loss = loss_f + loss_b + loss_c

        predictions = (ret_f['predictions'] + ret_b['predictions']) / 2
        imputations = (ret_f['imputations'] + ret_b['imputations']) / 2

        ret_f['loss'] = loss
        ret_f['predictions'] = predictions
        ret_f['imputations'] = imputations

        return ret_f

    def get_consistency_loss(self, pred_f, pred_b):
        loss = torch.pow(pred_f - pred_b, 2.0).mean()
        return loss

    def reverse(self, ret):
        def reverse_tensor(tensor_):
            if tensor_.dim() <= 1:
                return tensor_
            indices = range(tensor_.size()[1])[::-1]
            indices = Variable(torch.LongTensor(indices), requires_grad = False)

            if torch.cuda.is_available():
                indices = indices.cuda()

            return tensor_.index_select(1, indices)

        for key in ret:
            ret[key] = reverse_tensor(ret[key])

        return ret

    def run_on_batch(self, data, optimizer):
        ret = self(data)

        if optimizer is not None:
            optimizer.zero_grad()
            ret['loss'].backward()
            optimizer.step()

        return ret

==> imputegap-1.1.1-py3-none-any.whl: imputegap/wrapper/AlgoPython/BRITS/models/brits_i.py <==
import torch
import torch.nn as nn
from torch.autograd import Variable

from imputegap.wrapper.AlgoPython.BRITS.models import rits_i

SEQ_LEN = 36
RNN_HID_SIZE = 64


class Model(nn.Module):
    def __init__(self, batch_size, nbr_features, hidden_layers, seq_len):
        super(Model, self).__init__()
        self.batch_size = batch_size
        self.nbr_features = nbr_features
        self.hidden_layers = hidden_layers
        self.seq_len = seq_len
        self.build()

    def build(self):
        self.rits_f = rits_i.Model(self.batch_size, self.nbr_features, self.hidden_layers, self.seq_len)
        self.rits_b = rits_i.Model(self.batch_size, self.nbr_features, self.hidden_layers, self.seq_len)

    def forward(self, data):
        ret_f = self.rits_f(data, 'forward')
        ret_b = self.reverse(self.rits_b(data, 'backward'))

        ret = self.merge_ret(ret_f, ret_b)

        return ret

    def merge_ret(self, ret_f, ret_b):
        loss_f = ret_f['loss']
        loss_b = ret_b['loss']
        loss_c = self.get_consistency_loss(ret_f['imputations'], ret_b['imputations'])

        loss = loss_f + loss_b + loss_c

        predictions = (ret_f['predictions'] + ret_b['predictions']) / 2
        imputations = (ret_f['imputations'] + ret_b['imputations']) / 2

        ret_f['loss'] = loss
        ret_f['predictions'] = predictions
        ret_f['imputations'] = imputations

        return ret_f

    def get_consistency_loss(self, pred_f, pred_b):
        loss = torch.pow(pred_f - pred_b, 2.0).mean()
        return loss

    def reverse(self, ret):
        def reverse_tensor(tensor_):
            if tensor_.dim() <= 1:
                return tensor_
            indices = range(tensor_.size()[1])[::-1]
            indices = Variable(torch.LongTensor(indices), requires_grad = False)

            if torch.cuda.is_available():
                indices = indices.cuda()

            return tensor_.index_select(1, indices)

        for key in ret:
            ret[key] = reverse_tensor(ret[key])

        return ret

    def run_on_batch(self, data, optimizer):
        ret = self(data)

        if optimizer is not None:
            optimizer.zero_grad()
            ret['loss'].backward()
            optimizer.step()

        return ret
